# Copyright (c) 2014-present PlatformIO <contact@platformio.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from platformio.commands.remote.client.async_base import AsyncClientBase


class UpdateCoreClient(AsyncClientBase):
    def agent_pool_ready(self):
        d = self.agentpool.callRemote("cmd", self.agents, self.command, self.options)
        d.addCallback(self.cb_async_result)
        d.addErrback(self.cb_global_error)
